/* $Id: tdsvar.h,v 1.69 2002/03/13 06:35:10 rio Exp $ */

#ifndef _TDSVAR_H
#define _TDSVAR_H


#include <sys/types.h>
#include <sys/tdsio.h>
#include <sys/bus.h>


/* NIHON TSUSHINKI CO, LTD. Vender ID */
#define NITSUKI_VENDERID				0x7651		/* XXX temporary */

/* device ID */
#define NITSUKI_TIME_SYNC_DATA_SAMPLER	0x0002

/* device major */
#define TDS_MAJOR			222


struct tds_type {
    u_int16_t	tds_vid;
    u_int16_t	tds_did;
    char	*tds_name;
};


/* Registers (write) */
#define	TDS_REG_WRITE_SAMPLING_START		0x00
#define	TDS_REG_WRITE_FIFO_CLEAR		0x01
#define	TDS_REG_WRITE_INTERRUPT_SET		0x04
#define	TDS_REG_WRITE_SAMPLING_SET		0x05
#define	TDS_REG_WRITE_SYNCRONIZE		0x06
#define	TDS_REG_WRITE_TIME_SET			0x08
#define	TDS_REG_WRITE_DMA_ADDR_SET		0x10
#define	TDS_REG_WRITE_DMA_SIZE_SET		0x14
#define	TDS_REG_WRITE_DMA_START			0x18
#define	TDS_REG_WRITE_DMA_MODE			0x19
#define TDS_REG_WRITE_CLEAR_WRITE_COUNTER	0x20
#define TDS_REG_WRITE_CLEAR_READ_COUNTER	0x24


/* Registers (read) */
#define	TDS_REG_READ_DATA			0x00
#define	TDS_REG_READ_INTERRUPT			0x04
#define	TDS_REG_READ_STATUS			0x06
#define	TDS_REG_READ_YEAR			0x07
#define	TDS_REG_READ_TIME			0x08
#define	TDS_REG_READ_INPUT			0x0c
#define	TDS_REG_READ_DMA_SIZE			0x14
#define	TDS_REG_READ_DMA_STATUS			0x18
#define	TDS_REG_READ_DMA_MODE			0x19
#define	TDS_REG_READ_WRITE_COUNTER		0x20
#define	TDS_REG_READ_READ_COUNTER		0x24


/* interrupt */
#define TDS_INTR_1PPS			(0x01 << 0)
#define TDS_INTR_DMA_DONE		(0x01 << 1)
#define TDS_INTR_DATA_EXIST		(0x01 << 3)
#define TDS_INTR_DATA_EMPTY		(0x01 << 4)
#define TDS_INTR_DATA_FULL		(0x01 << 5)
#define TDS_INTR_DATA_HALF		(0x01 << 6)

/* dma mode */
#define TDS_DMA_MODE_SOFT		(0x00 << 0)
#define TDS_DMA_MODE_HARD		(0x01 << 0)
#define TDS_DMA_MODE_RELATIVE		(0x00 << 1)
#define TDS_DMA_MODE_REWIND		(0x01 << 1)

struct tds_softc;


struct tds_dma_buf {
    bus_dmamap_t	dmamap;
    void		*physaddr;
    void		*virtaddr;
    struct tds_softc	*sc;

    size_t		write_offset;
    size_t		read_offset;
    unsigned		flags;
};
typedef struct tds_dma_buf tds_dma_buf_t;


/* buffer flags */
#define TDS_BUFFLAG_READ_WAIT		(0x01 << 0)
#define TDS_BUFFLAG_DATA_READY		(0x01 << 1)
#define TDS_BUFFLAG_WRITE_BUSY		(0x01 << 2)

/* Hardware internal FIFO size */
#define TDS_FIFO_SIZE		(0x01 << 20)		/* 1M(Byte) */

/* max data size on busmaster transfer */
/* For soft trriger mode */
#define TDS_DMA_MAX_WORD	0x20000		/* 128K words */
#define TDS_DMA_MAX_SIZE	(TDS_DMA_MAX_WORD * sizeof(u_int32_t))
/* For hard trriger mode */
#define TDS_DMA_MAX_CHUNK	0x10000		/* 64K words */
#define TDS_DMA_MIN_CHUNK	0x00400		/*  1K words */

#define TDS_DMA_SIZE	TDS_FIFO_SIZE	/*(TDS_DMA_MAX_CHUNK * sizeof(u_int32_t))*/
#define	TDS_DMA_NSEG	8

#define	TDS_DMA_USE_CHUNK	0		/* dynamic */

/* data structure for each device */
struct tds_softc {
    dev_t	devt;
    device_t	devicet;
    int		unit;
    char	*dev_str;
    char	dev_name[8];

    struct resource	*res_io;
    struct resource	*res_irq;
    void	*cookie;
    u_int32_t	iobase;

    int		rev;
    int		irq;

    int		sampling_freq;
    int		sampling_width;
    int		sampling_channel;

    int		verbose;
    int		error;
    int		read_mode;
    int		max_fifo_overflow;
    u_int8_t	sampling_flag;
    u_int8_t	intr_flag;
    unsigned	is_open : 1;
    unsigned	sampling_done : 1;
    unsigned	pps_sync_done : 1;
    unsigned	time_sync_done : 1;
    unsigned	time_set_done : 1;
    unsigned	sampling_start : 1;

    /* synchronization variables */
    int		sync_1pps_count;	/* for 1PPS inerrupt */
    int		sync_data_count;	/* for DATA exist inerrupt */
    int		wait_buffer_ready;	/* for DMA Finish interrupt */
    int		fifo_underflow_count;	/* for FIFO empty iterrupt */
    int		fifo_overflow_count;	/* for FIFO full interrupt */
    int		fifo_overhalf_count;	/* for FIFO half interrupt */

    bus_dma_tag_t	dmatag;
    tds_dma_buf_t	dmabuf[TDS_DMA_NSEG];
    size_t		dmabuf_write;
    size_t		dmabuf_read;
    size_t		dma_buffer_size;	/* per byte */
    size_t		dma_chunk_size;		/* per byte/DMA */
};


#define TDSPRI		(PZERO - 1)
#define spltds()	spltty()
#define INTR_TYPE_TDS	INTR_TYPE_TTY


#define TDS_MAX_FIFO_OVERFLOW_DEFAULT	16
#define TDS_SAMPLING_DEFAULT	\
	(TDS_SAMPLING_40KHZ | TDS_SAMPLING_1BIT | TDS_SAMPLING_1CH)


/* FIFO counter */
#define TDS_FIFO_COUNTER_MAX    0x40000000
#define TDS_FIFO_COUNTER_MASK   0x3fffffff
#define TDS_FIFO_COUNTER_CLEAR  0x01


/* DMA control */
#define	TDS_DMA_START	0x01
#define	TDS_DMA_STOP	0x00

/* DMA status */
#define TDS_DMA_STATUS	0x01



#define TDS_TURN_ARROUND_MIN	100
#define TDS_TURN_ARROUND_MAX	4000

#endif	/* _TDSVAR_H */
